package edu.harvard.mgh.purcell.gplink.forms;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JTabbedPane;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import edu.harvard.mgh.purcell.gplink.base_form.*;
import edu.harvard.mgh.purcell.gplink.mainGUI.*;

@SuppressWarnings("serial")
public class MergeForm extends Forms {

	
	TextSelectPanel targetText;
	BinarySelectPanel targetBin;
	TXTSelectPanel targetTXT;
	JTabbedPane mergeInputs;
	
	//the various options
	private Forms.myRadioButton recode, recodeAD, recode12, recodeHV, makeBin;
	private JPanel recodeOp;
	
	private Forms.myRadioButton op1, op2, op3, op4, op5, op6, op7;
	private JPanel optional;
	
	public MergeForm(MainFrame mf) {
		super(mf, "Merge Filesets");
		
		this.buttons.disableFT();
		
	}
	
	@Override
	protected void initalize() {		
		targetBin = new BinarySelectPanel(this);
		targetText = new TextSelectPanel(this);
		targetTXT = new TXTSelectPanel(this);
		
		mergeInputs = new JTabbedPane();
		
		recodeOp = new JPanel();
		recode = new myRadioButton("Standard fileset (--recode)");
		recodeAD = new myRadioButton("Raw genotype file (--recodeAD)");
		recode12 = new myRadioButton("Standard fileset w/ allele recoding (--recode12)");
		recodeHV = new myRadioButton("Haploview fileset (--recodeHV)");
		makeBin = new myRadioButton("Binary fileset (--make-bed)");
		
		optional = new JPanel();
		op1 = new myRadioButton("1) Consensus call (default)");
		op2 = new myRadioButton("2) Only overwrite calls missing in original");
		op3 = new myRadioButton("3) Only overwrite calls not missing in new");
		op4 = new myRadioButton("4) Never overwrite");
		op5 = new myRadioButton("5) Always overwrite");
    	op6 = new myRadioButton("6) Report all mismatches");
    	op7 = new myRadioButton("7) Report non-missing mismatches");
	}

	void createMergeTarget(){
		ChangeListener checkMergeTab = new ChangeListener(){
			public void stateChanged(ChangeEvent e) {
				isValidCommand();
				op6.setEnabled(mergeInputs.getSelectedIndex() != 2);
				op7.setEnabled(mergeInputs.getSelectedIndex() != 2);
				
			}
		};
		
		mergeInputs.addChangeListener(checkMergeTab);
		mergeInputs.addTab("Merge Binary Fileset", targetBin);
		mergeInputs.addTab("Merge Standard Fileset", targetText);
		mergeInputs.addTab("Merge Multiple Filesets", targetTXT);
	}
	
	void createRecodeOps(){
		
		recodeOp.setLayout(new BoxLayout(recodeOp, BoxLayout.PAGE_AXIS));
		
		recode.setSelected(true);
		
		recodeOp.add(recode);
		recodeOp.add(recode12);
		recodeOp.add(recodeAD);
		recodeOp.add(recodeHV);
		recodeOp.add(makeBin);
		
		ButtonGroup recodeGroup = new ButtonGroup();
		recodeGroup.add(recode);
		recodeGroup.add(recode12);
		recodeGroup.add(recodeAD);
		recodeGroup.add(recodeHV);
		recodeGroup.add(makeBin);
		
	}
	
	void createOptions(){
		optional.setLayout(new BoxLayout(optional, BoxLayout.PAGE_AXIS));
		
    	ButtonGroup optionGroup = new ButtonGroup();
    	optionGroup.add(op1);
    	op1.setSelected(true);
    	optionGroup.add(op2);
    	optionGroup.add(op3);
    	optionGroup.add(op4);
    	optionGroup.add(op5);
    	optionGroup.add(op6);
    	optionGroup.add(op7);
    	
    	optional.add(op1);
    	optional.add(op2);
    	optional.add(op3);
    	optional.add(op4);
    	optional.add(op5);
    	optional.add(op6);
    	optional.add(op7);
    	
	}
	
	@Override
	protected void createBody() {
		body = new JPanel();
		body.setLayout(new BoxLayout(body, BoxLayout.PAGE_AXIS));
			
		createOptions();
		createMergeTarget();
		createRecodeOps();
		
		body.add(mergeInputs);
		JPanel formating = new JPanel();
		formating.setLayout(new BoxLayout(formating, BoxLayout.LINE_AXIS));
		formating.add(recodeOp);
		formating.add(new JSeparator(SwingConstants.VERTICAL));
		formating.add(optional);
		body.add(formating);
				
	}

	@Override
	protected String processBody() {
		String ans = "";
				
		if(mergeInputs.getSelectedIndex() == 0)
			ans += "--bmerge " 
				+ targetBin.getCommand(true);
		if(mergeInputs.getSelectedIndex() == 1)
			ans += "--merge " 
				+ targetText.getCommand(true);
		if(mergeInputs.getSelectedIndex() == 2)
			ans += "--merge-list " + targetTXT.getCommand();
		
		if(op2.isSelected() && op2.isEnabled())
			ans += " --merge-mode 2";
		if(op3.isSelected() && op3.isEnabled())
			ans += " --merge-mode 3";
		if(op4.isSelected() && op4.isEnabled())
			ans += " --merge-mode 4";
		if(op5.isSelected() && op5.isEnabled())
			ans += " --merge-mode 5";
		if(op6.isSelected() && op6.isEnabled())
			ans += " --merge-mode 6";
		if(op7.isSelected() && op7.isEnabled())
			ans += " --merge-mode 7";
		
		if(recode.isSelected())
			ans += " --recode";
		if(recodeAD.isSelected())
			ans += " --recodeAD";
		if(recode12.isSelected())
			ans += " --recode12";
		if(recodeHV.isSelected())
			ans += " --recodeHV";
		if(makeBin.isSelected())
			ans += " --make-bed";
		
		return ans;
	}

	@Override
	protected boolean isBodyValid() {
		if(((mergeInputs.getSelectedIndex() == 0) && !targetBin.validInput)
			||	((mergeInputs.getSelectedIndex() == 1) && !targetText.validInput)
			|| ((mergeInputs.getSelectedIndex() == 2) && !targetTXT.validInput )){
//			information.setText("Enter valid target for merging");
			return false;
		}
		
		if( !(recode.isSelected() || recodeAD.isSelected() 
				|| recodeHV.isSelected() || recode12.isSelected() 
				|| makeBin.isSelected()) ){
//			information.setText("Must select a coding option.");
			return false;
		}
		
		return true;
	}

	

}
