package edu.harvard.mgh.purcell.gplink.mainGUI;

import java.awt.BorderLayout;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.Vector;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JSplitPane;
import javax.swing.UIManager;

import edu.harvard.mgh.purcell.gplink.data.*;
/**
 * An extention of JFrame that contians the main window for this GUI.
 * @author Kathe Todd-Brown
 * 
 */
@SuppressWarnings("serial")
public class MainFrame extends JFrame {

	public static String VS = "0.291";
	//vs 0.291 haploview now browses locally, corrected population
	//...of file fields, add spacers to pop-up menu, add status bar to
	//...bottom of main window to track up/downloads
	//vs 0.290 added flags for incomplete downloads when closing
	//...MainFrame and trying to open files ---- POSTED 5/21/2007
	//vs 0.289 fixed bug in project dialog and fixed
	//...replaceHome to work with remote directories and
	//...resloved "String.replace" error ---- POSTED 5/18/2007
	//vs 0.288 project dialog now checks fields for done
	//vs 0.287 merge file fix
	//vs 0.286 improve no-reconnect selesction with connection lost
	//we no longer cycle through the dialogs endlessly
	//vs 0.285 added threading for up/downloads and remote excutions
	//vs 0.284 changed how files are copied... bugs fixed
	//vs 0.283 error dialog limited length
	//vs 0.282 fixed bug
	//vs 0.281 prettify the herietary dialog
	//vs 0.280 fixed bug
	//vs 0.279 fixed bugs and hopefully speed up the remote mode
	//vs 0.278 add option to import non-PLINK operation
	//...also optimized outfile validation
	//vs 0.277 add info file icon to optree
	//vs 0.276 checks that the file is actually deleted 
	//...when gplink tries to delete it
	//vs 0.275 group files together in heirachy
	//vs 0.274 add heirachy for operations
	
	public Project data;
	
	public FolderView folderViewer;

	public OpView operationsView;

	public LogView logViewer;
	
	public GPLINKMenuBar menuStuff;

	public Object lock = new Object();
	
	private Vector <String> filesInTransit;
	
	public JLabel statusBar;
	
	synchronized public void changeTransitFile(String [] files, 
			boolean addFile){
		if(addFile == true){
			for(String s: files){
				filesInTransit.add(s);
			}
		} else {
			for(String s: files){
				filesInTransit.remove(s);
			}
		}
		
		if(filesInTransit.isEmpty())
			statusBar.setText("Ready!");
		else{
			String message = "Down/Uploading: ";
			for(String file: filesInTransit){
				message = message + " [" + file + "]";
			}
			statusBar.setText(message);
			}
		
	}
	
	public boolean inTransit(String f){
		return filesInTransit.contains(f);
	}
	
	public void updateViews() {
		//System.out.println("Entering updageViews()@MainFrame.java "
		//		+ (new GregorianCalendar()).getTimeInMillis());
		data.updateFiles();
		operationsView.updateView();
		folderViewer.updateView();
		logViewer.update(logViewer.getCurrentFile(), true);
		//System.out.println("Exiting updageViews()@MainFrame.java " 
		//		+ (new GregorianCalendar()).getTimeInMillis());
	}

	public void updateViews(boolean forceUpdate) {
		//System.out.println("Entering updageViews("
		//		+forceUpdate+")@MainFrame.java  "
		//		+ (new GregorianCalendar()).getTimeInMillis());
		data.updateFiles(forceUpdate);
		operationsView.updateView();
		folderViewer.updateView();
		logViewer.update(logViewer.getCurrentFile(), true);
		//System.out.println("Exiting updageViews("
		//		+forceUpdate+")@MainFrame.java  "
		//		+ (new GregorianCalendar()).getTimeInMillis());
	}
	
	WindowListener properClosing = new WindowListener() {

		public void windowOpened(WindowEvent e) {}

		public void windowClosing(WindowEvent e) {
			if(filesInTransit.isEmpty() == false){
				String filelist = "";
				for(String file: filesInTransit){
					filelist += " [" + file + "] \n ";
				}
				
				int choice = JOptionPane.showConfirmDialog(MainFrame.this,
						"There are several files still being up/down loaded: \n"
						+filelist + "continuing will result in incomplete files. \n" +
						" Continue?", "gPLINK",
						JOptionPane.YES_NO_CANCEL_OPTION, 
						JOptionPane.INFORMATION_MESSAGE);
				if(choice != JOptionPane.YES_OPTION){
					return;
				}
			}
			data.closeOldProject();
			System.exit(0);
			
		}

		public void windowClosed(WindowEvent e) {}
		public void windowIconified(WindowEvent e) {}
		public void windowDeiconified(WindowEvent e) {}
		public void windowActivated(WindowEvent e) {}
		public void windowDeactivated(WindowEvent e) {}

	};

	public MainFrame() {
		super("gPLINK");
		
		data = new Project(null, this);
		operationsView = new OpView(this);
		logViewer = new LogView(this);
		folderViewer = new FolderView(this);
		menuStuff = new GPLINKMenuBar(this);
		filesInTransit = new Vector<String>();
		statusBar = new JLabel("Ready!");
		
		setJMenuBar(menuStuff);
		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		
		JSplitPane splitPaneHorz = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
                operationsView, logViewer);
		
		JSplitPane splitPaneVert = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
				folderViewer, splitPaneHorz);
		
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(splitPaneVert, BorderLayout.CENTER);
		getContentPane().add(statusBar, BorderLayout.PAGE_END);
		splitPaneVert.setDividerLocation(0.2);
		splitPaneHorz.setDividerLocation(0.8);
		this.addWindowListener(properClosing);
	}
	
	/**
	 * actually generates the main frame gui.
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		try {
	        UIManager.setLookAndFeel(
	        		//UIManager.getCrossPlatformLookAndFeelClassName()
	        		UIManager.getSystemLookAndFeelClassName()
	        		);
	    } catch (Exception e) { 
	    	System.err.println("Couldn't set look and feel, using default: MainFrame.java");
	    }

		
		MainFrame f = new MainFrame();
		f.pack();
		f.setVisible(true);

	}

}
